/*-
 * Copyright (c) 1997 Antti Kaipila
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include "cdplay.h"

extern int    term_fd;
extern char   *message,*UP,*BC,*up_line;
extern struct termios old_ttyattr;
extern struct cd_info cdinfo;

void check_ws(void)
{
  struct winsize ws;
  if(ioctl(STDIN_FILENO,TIOCGWINSZ,&ws)==-1) {
    fprintf(stderr,"cdplay: Cant get terminal dimesions\n");
    return;
  }
  if(ws.ws_col < 40 || ws.ws_row < 7){
    fprintf(stderr,"cdplay: Too small terminal dimesions\n");
    return;
  }
}

void print_head(void)
{
  printf(" ---------------------------------------\n");
}

void print_tail(void)
{
  printf(" | cdplay - %-20.20s  Quit |\n",message);
  printf(" | +----------+  Play      Stop        |\n");
  printf(" | | %2.2d %2.2d:%2.2d |  SPC pause Eject   +/- |\n",cdinfo.c_track,cdinfo.c_min,cdinfo.c_sec);
  printf(" | +----------+  Rew       Forward     |\n");
  printf(" ---------------------------------------\n");
  message="";
}

void move_up(int lines)
{
  int i;
  for(i=0;i<lines;i++) {
    tputs(tgoto(up_line,1,1),0,putchar);
  }  
}

int init_term(void)
{
  char term_buffer[2048];
  char *buffer=(char *)malloc(2048);
  struct termios ttyattr;

  check_ws();

  if(tgetent(term_buffer,getenv("TERM"))<1)
    return(-1);
  if((up_line=tgetstr("up",&buffer))==NULL)
    return(-1);
  UP=0;
  BC=0;

  tcgetattr(term_fd,&ttyattr);
  old_ttyattr=ttyattr;
  ttyattr.c_lflag &= ~(ECHO|ECHONL|ICANON);
  ttyattr.c_cc[VMIN]=1;
  ttyattr.c_cc[VTIME]=0;
  if(tcsetattr(term_fd,TCSAFLUSH,&ttyattr)<0)
    return(-1);
  return(0);
}


